<?php
// If this file is called directly, abort.
if ( ! defined( 'ABSPATH' ) ) {
    die();
}

/**
 * SEO Cleaner Core Plugin Class
 */
class SEO_Cleaner {

    /**
     * Plugin settings
     */
    public static $settings = false;
    /**
     * Default redirect status
     */
    public $redirect_status = 301;

    /**
     * Hook into the appropriate actions when the class is constructed
     */
    public function __construct() {

        self::$settings = array(
            'head'       => get_option( 'seocl_settings_head' ),
            'feed'       => get_option( 'seocl_settings_feed' ),
            'http'       => get_option( 'seocl_settings_http' ),
            'api'        => get_option( 'seocl_settings_api' ),
            'comments'   => get_option( 'seocl_settings_comments' ),
            'archives'   => get_option( 'seocl_settings_archives' ),
            'attachment' => get_option( 'seocl_settings_attachment' ),
        );

        if ( is_admin() ) {

            require_once SEOCL_PLUGIN_DIR . 'admin/class-seo-cleaner-admin.php';
            new SEO_Cleaner_Admin();

        }

        //hook loading order fix
        if( isset( self::$settings['head']['remove_migrate'] ) AND self::$settings['head']['remove_migrate'] == '1' ) {

            $this->remove_migrate();

        }

        add_action( 'init', array( $this, 'apply_settings' ) );

    }

    /**
     * Redirect function
     */
    public function redirect( $url = false ){

	    if( $url ) {
		    $target = $url;
	    } else {
		    $target = get_option( 'siteurl' );
	    }

        $target = apply_filters( 'seo_cleaner_redirect_target', $target );
        $status = apply_filters( 'seo_cleaner_redirect_status', $this->redirect_status );

        wp_redirect( $target, $status );
        die();

    }

    /**
     * Apply plugin settings
     */
    public function apply_settings() {

        if ( empty( self::$settings ) ) {
            return '';
        }

        self::$settings = apply_filters( 'seo_cleaner_settings', self::$settings );
        do_action( 'seo_cleaner_apply_settings', self::$settings );

        foreach ( self::$settings as $section ) {

            if( !empty( $section ) ) {

                foreach ( $section as $action => $status ) {

                    if ( $status == '1' && method_exists( __CLASS__, $action ) ) {

                        call_user_func( array( $this, $action ) );

                    }

                }

            }

        }

    }

    /**
     * Remove meta name=generator
     */
    public function remove_wp_version() {

        remove_action( 'wp_head', 'wp_generator' );
        add_filter( 'revslider_meta_generator', array( $this, 'remove_revslider_metadata' ) );
        $this->remove_visual_composer_metadata();

    }

    /**
    * Remove link rel=wlwmanifest
    */
    public function remove_wlwmanifest() {

        remove_action( 'wp_head', 'wlwmanifest_link' );

    }

    /**
    * Remove link rel=EditURI
    */
    public function remove_rsd() {

        remove_action( 'wp_head', 'rsd_link' );

    }

    /**
    * Remove link rel=shortlink
    */
    public function remove_shortlink() {

        remove_action( 'wp_head', 'wp_shortlink_wp_head' );

    }

    /**
    * Remove link rel=prev,next
    */
    public function remove_prev_next() {

        remove_action( 'wp_head', 'start_post_rel_link', 10 );
        remove_action( 'wp_head', 'adjacent_posts_rel_link_wp_head', 10 );

    }

    /**
    * Remove styles for .recentcomments
    */
    public function remove_recentcomments() {

        global $wp_widget_factory;
        remove_action( 'wp_head', array( $wp_widget_factory->widgets['WP_Widget_Recent_Comments'], 'recent_comments_style' ) );

    }

    /**
    * Remove Emoji Styles and Scripts
    */
    public function remove_emoji() {

        remove_action( 'wp_head', 'print_emoji_detection_script', 7 );
        remove_action( 'admin_print_scripts', 'print_emoji_detection_script' );
        remove_action( 'embeded_head', 'print_emoji_detection_script' );
        remove_action( 'wp_print_styles', 'print_emoji_styles' );
        remove_action( 'admin_print_styles', 'print_emoji_styles' );

    }

    /**
     * Remove WP-Embed
     */
    public function remove_embed() {

        wp_deregister_script( 'wp-embed' );

    }

    /**
     * Remove jQuery-Migrate
     */
    public function remove_migrate() {

        add_action( 'wp_default_scripts', array( $this, '_remove_migrate' ) );

    }

    public function _remove_migrate( $scripts ) {

        if ( !is_admin() && !empty( $scripts->registered['jquery'] ) ) {
            $jquery_dependencies = $scripts->registered['jquery']->deps;
            $scripts->registered['jquery']->deps = array_diff( $jquery_dependencies, array( 'jquery-migrate' ) );
        }

    }

    /**
    * Remove Feed Links
    */
    public function remove_feed() {

        remove_action( 'wp_head', 'feed_links_extra', 3 );
        remove_action( 'wp_head', 'feed_links', 2 );
        remove_action( 'wp_head', 'wc_products_rss_feed' );

    }

    /**
     * Disable Feeds
     */
    public function redirect_feed() {

        $this->remove_feed();

        add_action( 'do_feed', array( $this, '_redirect_feed' ), 1 );
        add_action( 'do_feed_rdf', array( $this, '_redirect_feed' ), 1 );
        add_action( 'do_feed_rss', array( $this, '_redirect_feed' ), 1 );
        add_action( 'do_feed_rss2', array( $this, '_redirect_feed' ), 1 );
        add_action( 'do_feed_atom', array( $this, '_redirect_feed' ), 1 );

    }

    public function _redirect_feed() {

        $this->redirect();

    }

    /**
     * Remove Link rel=shortlink from http
     */
    public function remove_server_shortlink() {

        remove_action( 'template_redirect', 'wp_shortlink_header', 11 );

    }

    /**
     * Remove X-Pingback
     */
    public function remove_pingback() {

        add_filter( 'wp_headers', array( $this, '_remove_pingback' ) );
        add_action( 'wp', array( $this, '__remove_pingback' ) );

    }

    public function _remove_pingback( $headers ) {

        if( isset( $headers['X-Pingback']) ) {
            unset( $headers['X-Pingback'] );
        }

        return $headers;

    }

    public function __remove_pingback() {

        if ( function_exists( 'header_remove' ) ) {
            header_remove( 'X-Pingback' );
        }

    }

    /**
     * Remove X-Powered-By
     */
    public function remove_powered() {

        if ( function_exists( 'header_remove' ) ) {
            header_remove( 'x-powered-by' );
        }

        add_action( 'wp', array( $this, '_remove_powered' ) );

    }

    public function _remove_powered() {

        if ( function_exists( 'header_remove' ) ) {
            header_remove( 'x-powered-by' );
        }

    }

    /**
    * Remove WP API Links and Scripts
    */
    public function remove_api_head() {

        remove_action( 'wp_head', 'rest_output_link_wp_head' );
        remove_action( 'wp_head', 'wp_oembed_add_discovery_links' );

    }

    /**
    * Remove WP API Link from HTTP Headers
    */
    public function remove_api_server() {

        remove_action( 'template_redirect', 'rest_output_link_header', 11 );

    }

    /**
    * Totally Disable WP API Feature
    */
    public function disable_api() {

        $this->remove_api_server();
        $this->remove_api_head();

        add_filter( 'json_enabled', '__return_false' );
        add_filter( 'json_jsonp_enabled', '__return_false' );
        add_filter( 'rest_enabled', '__return_false' );
        add_filter( 'rest_jsonp_enabled', '__return_false' );

    }

    /**
     * Remove Website/URL field
     */
    public function remove_comments_url() {

        add_filter( 'comment_form_default_fields', array( $this, '_remove_comments_url' ) );

    }

    public function _remove_comments_url( $fields ) {

        if( isset( $fields['url'] ) )
            unset( $fields['url'] );

        return $fields;

    }

    /**
     * Remove comment notes
     */
    public function remove_comments_notes() {

        add_filter( 'comment_form_defaults', array( $this, '_remove_comments_notes' ) ) ;

    }

    public function _remove_comments_notes( $defaults ) {

        $defaults['comment_notes_before'] = '';

        return $defaults;

    }

    /**
     * Remove comment author link
     */
    public function remove_comments_author_link() {

        add_filter( 'get_comment_author_link', array( $this, '_remove_comments_author_link' ) ) ;

    }

    public function _remove_comments_author_link( $author_link ) {

        return strip_tags( $author_link );

    }

    /**
     * Remove comment auto linking
     */
    public function remove_comments_autolinking() {

        remove_filter( 'comment_text', 'make_clickable', 9 );

    }

    /**
     * Remove date archives
     */
    public function remove_archives_date() {

        add_action( 'template_redirect', array( $this, '_remove_archives_date' ) );

    }

    public function _remove_archives_date() {

        if( is_date() )
            $this->redirect();

    }

    /**
     * Remove author archives
     */
    public function remove_archives_author() {

        add_action( 'template_redirect', array( $this, '_remove_archives_author' ) );

    }

    public function _remove_archives_author() {

        if( is_author() )
            $this->redirect();

    }

    /**
     * Remove tag archives
     */
    public function remove_archives_tag() {

        add_action( 'template_redirect', array( $this, '_remove_archives_tag' ) );

    }

    public function _remove_archives_tag() {

        if( is_tag() )
            $this->redirect();

    }

    /**
     * Remove category archives
     */
    public function remove_archives_category() {

        add_action( 'template_redirect', array( $this, '_remove_archives_category' ) );

    }

    public function _remove_archives_category() {

        if( is_category() )
            $this->redirect();

    }

    /**
     * Remove category archives
     */
    public function remove_archives_postformat() {

        add_action( 'template_redirect', array( $this, '_remove_archives_postformat' ) );

    }

    public function _remove_archives_postformat() {

        if( is_tax( 'post_format' ) )
            $this->redirect();

    }

    /**
     * Remove search
     */
    public function remove_archives_search() {

        add_action( 'template_redirect', array( $this, '_remove_archives_search' ) );

    }

    public function _remove_archives_search() {

        if( is_search() )
            $this->redirect();

    }

    /**
     * Remove attachments
     */
    public function remove_attachment() {

        add_action( 'template_redirect', array( $this, '_remove_attachment' ) );

    }

    public function _remove_attachment() {

        if ( is_attachment() ){

			global $post;

	        $url = get_the_permalink( $post->post_parent );

	        $this->redirect( $url );

        }

    }

    /**
     * Remove DNS Prefetch Links
     */
    public function remove_dns_prefetch() {

        remove_action( 'wp_head', 'wp_resource_hints', 2 );

    }

    /**
     * Remove Visual Composer Generator
     */
    public function remove_visual_composer_metadata() {

        global $vc_manager;

        if ( $vc_manager ) {
            remove_action( 'wp_head', array( $vc_manager->vc(), 'addMetaData' ) );
        }

    }

    /**
     * Remove Slider Revolution Generator
     */
    public function remove_revslider_metadata() {

        return '';

    }

}